@@ -1,7 +1,6 @@
 # -*- coding: UTF-8 -*-
 
-from pymta.authenticator import *
-from pymta.default_policy import *
+from pymta.api import *
 from pymta.model import *
 from pymta.mta import *
 from pymta.command_parser import *
